package com.fjhx.wms.service.stock.impl;

import cn.hutool.core.util.ObjectUtil;
import com.fjhx.item.entity.product.po.ProductInfo;
import com.fjhx.item.service.product.ProductInfoService;
import com.fjhx.wms.entity.stock.dto.StockJournalDetailsDto;
import com.fjhx.wms.entity.stock.po.Stock;
import com.fjhx.wms.entity.stock.po.StockJournal;
import com.fjhx.wms.entity.stock.po.StockJournalDetails;
import com.fjhx.wms.mapper.stock.StockMapper;
import com.fjhx.wms.service.stock.StockJournalDetailsService;
import com.fjhx.wms.service.stock.StockJournalService;
import com.fjhx.wms.service.stock.StockService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.wms.entity.stock.dto.StockSelectDto;
import com.obs.services.internal.ServiceException;
import com.ruoyi.common.core.domain.BaseSelectDto;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.fjhx.wms.entity.stock.vo.StockVo;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.fjhx.wms.entity.stock.dto.StockDto;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;


/**
 * <p>
 * 库存 服务实现类
 * </p>
 *
 * @author
 * @since 2023-03-20
 */
@Service
public class StockServiceImpl extends ServiceImpl<StockMapper, Stock> implements StockService {
    @Autowired
    StockJournalService stockJournalService;
    @Autowired
    StockJournalDetailsService stockJournalDetailsService;
    @Autowired
    ProductInfoService productInfoService;

    @Override
    public Page<StockVo> getPage(StockSelectDto dto) {
        IWrapper<Stock> wrapper = getWrapper();
        wrapper.orderByDesc("s", Stock::getId);
        wrapper.eq(Stock::getWarehouseId, dto.getId());
        wrapper.like(Stock::getQuantity, dto.getKeyword());

        Page<StockVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        List<StockVo> stockVos = page.getRecords();
        //获取产品id列表
        List<Long> productIds = stockVos.stream().map(Stock::getProductId).collect(Collectors.toList());
        if (ObjectUtil.isNotEmpty(productIds)) {
            List<ProductInfo> productInfos = productInfoService.listByIds(productIds);
            Map<Long, ProductInfo> productInfoMap = productInfos.stream().collect(Collectors.groupingBy(ProductInfo::getId,
                    Collectors.collectingAndThen(Collectors.toList(), value -> value.get(0))));
            for (StockVo stockVo : stockVos) {
                ProductInfo productInfo = productInfoMap.get(stockVo.getProductId());
                stockVo.setType(productInfo.getType());
                stockVo.setProductName(productInfo.getName());
            }
        }

        return page;
    }

    @Override
    public Stock detail(StockDto dto) {
        return getOne(q->q.eq(Stock::getWarehouseId,dto.getWarehouseId()).eq(Stock::getProductId,dto.getProductId()));
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void add(StockDto stockDto) {
        //创建出入库记录
        StockJournal stockJournal = new StockJournal();
        stockJournal.setType(stockDto.getType());
        stockJournal.setWarehouseId(stockDto.getWarehouseId());
        stockJournal.setBusinessId(stockDto.getBusinessId());
        //保存出入库记录
        stockJournalService.save(stockJournal);
        //操作库存
        List<StockJournalDetails> stockJournalDetailsList = ModifyInventory(stockJournal.getId(), 1, stockDto.getList(), stockDto.getWarehouseId());
        //保存出入库明细
        stockJournalDetailsService.saveBatch(stockJournalDetailsList);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void edit(StockDto stockDto) {
        //创建出入库记录
        StockJournal stockJournal = new StockJournal();
        stockJournal.setType(stockDto.getType());
        stockJournal.setWarehouseId(stockDto.getWarehouseId());
        stockJournal.setBusinessId(stockDto.getBusinessId());
        //保存出入库记录
        stockJournalService.save(stockJournal);
        //操作库存
        List<StockJournalDetails> stockJournalDetailsList = ModifyInventory(stockJournal.getId(), 2, stockDto.getList(), stockDto.getWarehouseId());
        //保存出入库明细
        stockJournalDetailsService.saveBatch(stockJournalDetailsList);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void stockTransfer(StockDto stockDto) {
        //创建出入库记录
        StockJournal stockJournal = new StockJournal();
        stockJournal.setType(3);
        stockJournal.setWarehouseId(stockDto.getWarehouseId());
        stockJournal.setToWarehouseId(stockDto.getToWarehouseId());
        //保存出入库记录
        stockJournalService.save(stockJournal);
        //操作库存
        List<StockJournalDetails> stockJournalDetailsList = ModifyInventory(stockJournal.getId(), 2, stockDto.getList(), stockDto.getWarehouseId());
        ModifyInventory(stockJournal.getId(), 1, stockDto.getList(), stockDto.getToWarehouseId());
        //保存出入库明细
        stockJournalDetailsService.saveBatch(stockJournalDetailsList);
    }

    @Override
    public void delete(Long id) {
        this.removeById(id);
    }

    /**
     * 操作库存通用方法
     *
     * @param stockJournalId 出入库记录id
     * @param type           1入库 2出库
     */
    @Override
    public List<StockJournalDetails> ModifyInventory(Long stockJournalId, int type, List<Stock> list, Long warehouseId) {
        List<Stock> data = new ArrayList<>();

        //获取现有库存
        List<Long> productIds = list.stream().map(Stock::getProductId).collect(Collectors.toList());
        List<Stock> stockList = list(q -> q.in(Stock::getProductId, productIds).eq(Stock::getWarehouseId, warehouseId));
        Map<Long, Stock> stockMap = stockList.stream().collect(Collectors.groupingBy(Stock::getProductId,
                Collectors.collectingAndThen(Collectors.toList(), value -> value.get(0))));

        List<StockJournalDetails> stockJournalDetailsList = new ArrayList<>();
        for (Stock stock : list) {
            Stock newStocks = new Stock();
            newStocks.setWarehouseId(warehouseId);
            newStocks.setProductId(stock.getProductId());
            newStocks.setQuantity(stock.getQuantity());

            //合并库存数量
            Stock oldStocks = stockMap.get(stock.getProductId());
            //用来存储操作数据
            if (ObjectUtil.isNotEmpty(oldStocks)) {
                BigDecimal quantity = oldStocks.getQuantity();
                if (type == 1) {
                    //入库库存相加
                    quantity = quantity.add(stock.getQuantity());
                } else if (type == 2) {
                    //出库库存相减
                    quantity = quantity.subtract(stock.getQuantity());
                    if (quantity.compareTo(BigDecimal.ZERO) < 0) {
                        throw new ServiceException("库存不足无法出库");
                    }
                } else {
                    throw new ServiceException("未知库存操作类型");
                }
                newStocks.setId(oldStocks.getId());
                newStocks.setQuantity(quantity);
            } else if (type == 2) {
                throw new ServiceException("以下商品库存不存在，无法出库：" + stock.getProductId());
            }

            data.add(newStocks);
            //创建出入库明细
            StockJournalDetails stockJournalDetails = new StockJournalDetailsDto();
            stockJournalDetails.setStockJournalId(stockJournalId);
            stockJournalDetails.setProductId(stock.getProductId());
            stockJournalDetails.setQuantity(stock.getQuantity());
            stockJournalDetailsList.add(stockJournalDetails);
        }
        //操作库存
        saveOrUpdateBatch(data);

        //返回库存操作明细列表
        return stockJournalDetailsList;
    }


}
